/* $Id: fxo_test.c 132 2006-01-27 23:48:21Z riz $ */

#define CHANNEL_DEV "/dev/zap/channel"

#include <sys/time.h>
#include <unistd.h>

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <zaptel.h>
#include <tonezone.h>

/*
 * List of named event
 */
static char *events[] = {
	"No event",
	"On hook",
	"Ring/Answered",
	"Wink/Flash",
	"Alarm",
	"No more alarm",
	"HDLC Abort",
	"HDLC Overrun",
	"HDLC Bad FCS",
	"Dial Complete",
	"Ringer On",
	"Ringer Off",
	"Hook Transition Complete",
	"Bits Changed",
	"Pulse Start"
};

#define READ_SIZE 160

static char *event2str(int event)
{
	static char buf[256];
	if ((event < 15) && (event > -1))
		return events[event];
	sprintf(buf, "Event %d", event);
	return buf;
}



/*
 * Set hook to given state
 */
int zt_hook(int fd, int x)
{
	int hook = x;
	return ioctl(fd, ZT_HOOK, &hook);
}

/*
 * Switch to onhook state
 */
void zt_onhook(int fd)
{
	if(zt_hook(fd, ZT_ONHOOK))
		perror("ioctl(zt_onhook)");
}

/*
 * Switch to offhook state
 */
void zt_offhook(int fd)
{
	if(zt_hook(fd, ZT_OFFHOOK))
		perror("ioctl(zt_offhook)");
}

/*
 * Set linear mode
 */
int zt_setlinear(int zfd, int linear)
{
	int res;
	res = ioctl(zfd, ZT_SETLINEAR, &linear);
	if (res)
		return res;
	return 0;
}

/*
 * Set ALAW/MULAW mode
 * law is one of ZT_LAW_ALAW/ZT_LAW_MULAW
 */
int zt_setlaw(int zfd, int law)
{
	int res;
	res = ioctl(zfd, ZT_SETLAW, &law);
	if (res)
		return res;
	return 0;
}

/*
 * Send one pulse
 */
void pulse(fd)
{
	// t1 - pulse time
	// t2 - pause between pulses
	// t1/t2 should be about 40/60
	// pulse frequency is 100 Hz
	zt_onhook(fd);
	usleep(40000);
	zt_offhook(fd);
	usleep(60000);
}

/*
 * ATDP digit :) 
 */

void dial_pulse(int fd, unsigned char digit)
{
	/* skip if it is something else then digit */
	if((digit < 1) || (digit > 10))
		return;

	while(digit)
	{
		pulse(fd);
		digit--;
	}
	// pause between pulses - 800 ms
	usleep(800000);
}




void zt_dial(int fd, char * dialstr)
{
	ZT_DIAL_OPERATION dop;
	dop.op = ZT_DIAL_OP_REPLACE;
	strncpy(dop.dialstr, dialstr, 20);
	if (ioctl(fd, ZT_DIAL, &dop)) {
			perror("ioctl(zt_dial)");
	}
}


/*
 * Get zaptel event from channel
 * -1 if ioctl fails
 */
int zt_get_event(int fd)
{
	int event;
	if(ioctl(fd, ZT_GETEVENT, &event) == -1)
		return -1;
	return event;
}

/*
 * exception  handler
 * The only its task - get ZT events 
 */
void zt_exception(int fd)
{
	int res;
	res = zt_get_event(fd);
	fprintf(stderr, "Got event '%s'(%d)\n", event2str(res), res);
	if(res == ZT_EVENT_RINGOFFHOOK)
	{
		zt_offhook(fd);
	}
}

/*
 * Open device then specify channe and blocksize
 */
int zt_open(int chan, int blocksize)
{
		int zfd;
		zfd = open(CHANNEL_DEV, O_RDWR | O_NONBLOCK);

		if(zfd < 0)
		{
				perror("open");
				return -1;
		}

		if(ioctl(zfd, ZT_SPECIFY, &chan) < 0)
		{
				perror("ioctl(ZT_SPECIFY)");
				close(zfd);
				return -1;
		}

		if(ioctl(zfd, ZT_SET_BLOCKSIZE, &blocksize) < 0)
		{
				perror("ioctl(ZT_BLOCK_SIZE)");
				close(zfd);
				return -1;
		}

		return zfd;
}

void usage()
{
		printf("Use: ./fxo_test on|off|dial\n");
}

int main(int argc, char ** argv)
{
		/* channel descriptor */
		int zfd, fd;

		/* fd sets for select() */
		fd_set wfds, rfds, efds;
		struct timeval	tv;
		ZT_PARAMS ps;

		if(argc < 2)
		{
				usage();
				return 0;
		}

		zfd = zt_open(1, READ_SIZE);
		if(zfd < 0)
		{
				return 1;
		}
		
		if(! strcmp(argv[1],"on"))
				zt_onhook(zfd);
		else if(! strcmp(argv[1],"off"))
		{
			zt_offhook(zfd);
			// zt_dial(zfd);
			// dial "2"
		}
		else if(! strcmp(argv[1],"dial"))
		{
			// zt_onhook(zfd);
			// sleep(5);
			zt_offhook(zfd);
			// zt_setlaw(zfd, 1); // mu-law
			zt_setlinear(zfd, 0); // we want linear sound
			if(argc < 3)
			{
				fprintf(stderr, "No string to dial\n");
				close(zfd);
				exit(1);
			}
			zt_dial(zfd, argv[2]);

		}
		else if(! strcmp(argv[1],"info"))
		{
			if(ioctl(zfd, ZT_GET_PARAMS, &ps) < 0 )
			{
				perror("ioctl");
				exit(1);
			}
			printf("LAW = %d\n", ps.curlaw);
		}
		else
				usage();
		fd = open("sound", O_RDWR | O_CREAT, 0664);
		// tone_zone_play_tone(zfd, ZT_TONE_BUSY);
		for(;;)
		{
			int res;

			FD_ZERO(&rfds);
			FD_ZERO(&wfds);
			FD_ZERO(&efds);
			FD_SET(zfd, &rfds);
			FD_SET(zfd, &wfds);
			FD_SET(zfd, &efds);
			tv.tv_sec = 5;
			tv.tv_usec = 0;
			res = select(zfd+1, &rfds, NULL, &efds, &tv);
			if(res > 0)
			{
				if(FD_ISSET(zfd, &efds))
				{
					zt_exception(zfd);
				}

				if(FD_ISSET(zfd, &rfds))
				{
					char buffer[READ_SIZE*2];
					int result;

					result = read(zfd, buffer, READ_SIZE*2);
					write(fd, buffer, result);
				}
			}

			/*
			if(FD_ISSET(zfd, &wfds))
				fprintf(stderr, "Write!");
				*/
		}
		close(zfd);
		return 0;
}
